-- print goes to sublime output
io.stdout:setvbuf("no")

require "Stack"

local labyrinth = {}
local imageFloor = false
local imageWall = false
local spriteSize = 0

function love.load()
	math.randomseed(os.time())
	loadImages()
	generateLabyrinth()
end

function love.mousepressed(x, y, button, istouch)
   generateLabyrinth()
end

function love.keyreleased(key)
	if key == "escape" then
    	love.event.quit()
   	else
		generateLabyrinth()
	end
end

function love.update(dt)
end

function love.draw()
	for x, _ in ipairs(labyrinth) do
		for y, _ in ipairs(labyrinth[1]) do
			love.graphics.draw(labyrinth[x][y].image, x*spriteSize, y*spriteSize)
		end
	end
end

function loadImages()
	imageFloor = love.graphics.newImage("assets/floor.png")
	imageWall = love.graphics.newImage("assets/wall.png")

	spriteSize = imageWall:getWidth()
end

function generateLabyrinth()
	initLabyrinth()
	linkNeighbor()
	generatePaths()
end

function initLabyrinth()
	local L_W = love.graphics.getWidth() / spriteSize
	local L_H = love.graphics.getHeight() / spriteSize

	for x=1, L_W do
		labyrinth[x] = {}

		for y=1, L_H do
			labyrinth[x][y] = 
			{
				isWall = true,
				N, E, S, W = false,
				image = imageWall,
			}
		end
	end
end

function linkNeighbor()
	for x=1, #labyrinth do
		for y=1, #labyrinth[1] do

			if y+1 <= #labyrinth[1] then labyrinth[x][y].N = labyrinth[x][y+1] end
			if x+1 <= #labyrinth then labyrinth[x][y].E = labyrinth[x+1][y] end
			if y-1 >= 1 then labyrinth[x][y].S = labyrinth[x][y-1] end
			if x-1 >= 1 then labyrinth[x][y].W = labyrinth[x-1][y] end
		end
	end
end

function generatePaths()
	
	local stack = Stack:create()

	local curr = labyrinth[3][1]
	makeFloor(curr)

	stack:push(curr)

	while not stack:empty() do

		local neighbors = GetNeighbors(curr)

		if #neighbors > 0 then

			local adj, nxt = pickRandomNeighbors(neighbors)
			makeFloor(adj)
			makeFloor(nxt)

			stack:push(curr)
			curr = nxt
		else
			curr = stack:pop()
		end
	end
end

function GetNeighbors(tile)
	neighbors = {}

	if tile.N and tile.N.N and tile.N.N.isWall then
		table.insert(neighbors, {tile.N, tile.N.N})
	end

	if tile.E and tile.E.E and tile.E.E.isWall then
		table.insert(neighbors, {tile.E, tile.E.E})
	end

	if tile.S and tile.S.S and tile.S.S.isWall then
		table.insert(neighbors, {tile.S, tile.S.S})
	end

	if tile.W and tile.W.W and tile.W.W.isWall then
		table.insert(neighbors, {tile.W, tile.W.W})
	end

	return neighbors
end

function pickRandomNeighbors(neighbors)
	i = math.random(1, #neighbors)
	return neighbors[i][1], neighbors[i][2]
end

function makeFloor(tile)
	tile.image = imageFloor
	tile.isWall = false
end